#!/usr/bin/env python
# -*- coding: UTF-8 -*-

'''
  ファイル名 : moo_cam.py

  カメラ画像取得クラス(UVC、Pi Camera)

  開発環境 : Raspberry Pi3 model B、Pi3 model B+、ZERO W
             Python 3.5.3

  2018/06 : Ver.0.1 初版

  Copyright 2018 Moo Soft(Akiyosi Yokoyama) https://moosoft.jp

  -------------------------------------------------------------------------
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
  -------------------------------------------------------------------------

  参考 URL
    FSWEBCAM USB Web カメラ
    sudo apt-get -y install fswebcam
    https://www.raspberrypi.org/documentation/usage/webcams/

    Picamera
    https://picamera.readthedocs.io/

  ＜日本語フォント＞
    sudo apt-get install fonts-noto
'''
import os.path
import subprocess
from datetime import datetime
import time
import picamera

# ============================================== #
# カメラ画像作成クラス
# ============================================== #
class CamImage(object):

    ''' PiCamera 撮影映像に適用される効果文字列一覧
    effect = ['none',
              'negative',
              'solarize',
              'sketch',
              'emboss',
              'oilpaint',
              'hatch',
              'gpen',
              'posterise'
             ]
    '''
    # ========================================== #
    # コンストラクタ
    # cam_w : カメラ画像幅
    # cam_h : カメラ画像高
    # ========================================== #
    def __init__(self, cam_w=1280, cam_h=720):
        self.cam_w = cam_w # カメラ解像度
        self.cam_h = cam_h

    # ========================================== #
    # カメラ画像作成(PiCamera, fswebcam  自動選択)
    # /dev/video? が存在すれば fswebcam 優先 
    # gpath  : カメラ画像ファイル名
    # ief    : PiCamera 撮影映像に適用される効果文字列
    # 戻り値 : 成功 True  それ以外 False
    # ========================================== #
    def get_image(self, gpath, ief='none'):
        result = False
        try:
            if os.path.isfile(gpath): os.remove(gpath) # ファイルがあれば削除
            dev = self.get_cmdresult('ls /dev/video?')
            print('dev : {0}'.format(dev))
            if 'video' in dev: result = self.image_uvc(gpath)
            else:              result = self.image_picam(gpath, ief)
        except Exception as e:
            print('get_image:{0}'.format(e))
        return result

    # ============================================== #
    # 標準出力取得
    # 戻り値 : 標準出力 エラー時 ''
    # ============================================== #
    def get_cmdresult(self, cmd):
        result = ''
        try:
            result = subprocess.check_output(cmd, shell=True).strip()
            result = result.decode('utf-8')
        except Exception as e:
            print('get_cmdresult:{0}'.format(e))
        return result

    # ========================================== #
    # カメラ画像作成(PiCamera 使用)
    # gpath  : カメラ画像ファイル名
    # ief    : 撮影映像に適用される効果文字列
    # 戻り値 : 成功 True  それ以外 False
    # ========================================== #
    def image_picam(self, gpath, ief='none'):
        result = False
        try:
            nowd = datetime.now()                                  # 日時取得
            nowt = nowd.strftime('%Y/%m/%d %H:%M:%S')              # 日時フォーマット
            with picamera.PiCamera() as cam:
                cam.resolution          = (self.cam_w, self.cam_h) # 解像度(デフォルト:1280x720)
                cam.framerate           = 24                       # フレームレート
                cam.image_effect        = ief
                cam.annotate_text_size  = 20                       # 6 to 160, inclusive. The default is 32.
                cam.annotate_foreground = picamera.Color('black')  # white or black デフォルト white
                cam.annotate_background = picamera.Color('white')  # デフォルト 透過
                cam.annotate_text = ' {0} '.format(nowt)           # タイムスタンプテキストを重ねる
                #cam.start_preview()                               # プレビュー
                #time.sleep(2)                                     # 待機
                cam.capture(gpath)                                 # カメラ画像出力(プレビュー無しでも可)
                #cam.stop_preview()                                # プレビュー停止
            result = os.path.isfile(gpath)                         # ファイルチェック
        except Exception as e:
            print('image_picam:{0}'.format(e))
        return result

    # ========================================== #
    # カメラ画像作成(fswebcam 使用)
    # gpath  : カメラ画像ファイル名
    # 戻り値 : 成功 True  それ以外 False
    # ========================================== #
    def image_uvc(self, gpath):
        try:
            cmd = 'fswebcam -r {0}x{1} --skip 1 -F 20 --title "{2}" --font NotoMono-Regular.ttf:12 --text-colour "#00ffffff" --timestamp "%Y/%m/%d %H:%M:%S ( %Z )" {2}'
            cmd = cmd.format(self.cam_w, self.cam_h, gpath)
            for ii in range(0, 2, 1):                        # 2回実行...起動後最初の画像取得に失敗する事がある為の対策
                subprocess.call(cmd, timeout=10, shell=True) # カメラ画像作成
                #subprocess.call(cmd, shell=True)            # カメラ画像作成
                time.sleep(5.0)
        except Exception as e:
            print('image_uvc:{0}'.format(e))
        return os.path.isfile(gpath) # ファイル存在チェック

# end class CamImage

# ============================================== #
# メインループ
# ============================================== #
def main():
    try:
        CamImage().get_image('./tmp/cam.jpg')
        #CamImage().image_picam('./tmp/picam.jpg')
        #CamImage().image_uvc('./tmp/uvc.jpg')
    except Exception as e:
        print('main:{0}'.format(e))

# ============================================== #
if __name__ == "__main__":
    main()
